#!/bin/bash


function make_arch_package() {
    version=$1

    cat > PKGBUILD << EOF
pkgname=greadymedia
pkgver=$version
pkgrel=1
pkgdesc='ReadyMedia GTK frontend'
arch=('x86_64' 'aarch64')
url='https://fox-echo.info/greadymedia'
license=('CeCILL')
depends=('gtk3')
makedepends=('cmake' 'make' 'gcc')
source=(\$pkgname-\$pkgver.tar.gz)
md5sums=(SKIP)

build() {
    cd \$pkgname-\$pkgver
    mkdir -p build
    cd build
    cmake .. -DCMAKE_INSTALL_PREFIX=/usr -DCMAKE_INSTALL_DOCDIR=share/doc
    make
}

package() {
    cd \$pkgname-\$pkgver/build
    make install DESTDIR="\$pkgdir"
    mv "\$pkgdir"/usr/share/doc/\$pkgname-\$pkgver "\$pkgdir"/usr/share/doc/\$pkgname
    install -Dm644 "\$srcdir"/\$pkgname-\$pkgver/doc/License_CeCILL_V2.1-en.txt "\$pkgdir"/usr/share/licenses/\$pkgname/LICENSE
}

EOF
    makepkg -f
    rm -rf PKGBUILD src pkg greadymedia-debug-$version-1-*.pkg.tar.*
    mv greadymedia-$version-1-*.pkg.tar.* arch/
}


function make_slackware_package() {
    version=$1

    arch=$(uname -m); if ! echo $arch | grep -q "64"; then arch="i586"; fi
    tar -zxvf greadymedia-$version.tar.gz; cd greadymedia-$version
    mkdir -p build; cd build
    cmake .. -DCMAKE_INSTALL_PREFIX=/usr
    make; make install DESTDIR=pack
    strip --strip-unneeded pack/usr/bin/greadymedia

    mkdir pack/install
    cat > pack/install/slack-desc << EOF
greadymedia: greadymedia (ReadyMedia GTK frontend)
greadymedia:
greadymedia: gReadyMedia is a GTK frontend to setup a ReadyMedia (formerly
greadymedia: MiniDLNA - http://minidlna.sourceforge.net/) UPnp service.
greadymedia:
greadymedia: https://fox-echo.info/greadymedia
greadymedia:
EOF
    #Slackware 'makepkg' faker
    cd pack; for lnk in $(find -type l); do
        name=$(basename $lnk); dir=$(dirname $lnk | cut -c2-); target=$(basename $(readlink $lnk))
        echo -e "( cd $dir ; rm -rf $name )\n( cd $dir ; ln -sf $target $name )" >> install/doinst.sh
        rm -f $lnk
    done; cd ..
    sudo chown -R root:root pack
    cd pack; find ./ | LC_COLLATE=C sort | sed '2,$s,^\./,,' | tar --no-recursion -T - -cvf - | xz --threads=0 -c > ../../../slackware/greadymedia-$version-$arch-orig.txz; cd ..
    sudo chown -R $USER pack
    cd ../..; rm -rf greadymedia-$version
}


function make_debian_package() {
    version=$1

    arch=$(uname -m); if [ "$arch" == "x86_64" ]; then arch="amd64"; else arch="arm64"; fi
    tar -zxvf greadymedia-$version.tar.gz; cd greadymedia-$version
    mkdir -p build; cd build
    cmake .. -DCMAKE_INSTALL_PREFIX=/usr
    make; make install DESTDIR=pack
    strip --strip-unneeded pack/usr/bin/greadymedia
    mkdir pack/DEBIAN
    cat > pack/DEBIAN/control << EOF
Package: greadymedia
Version: $version
Architecture: any
Depends: libgtk-3-0
Maintainer: Frank Endres <frankendres@tuxfamily.org>
Description: ReadyMedia GTK frontend
Homepage: https://https://fox-echo.info/greadymedia/
EOF
    find pack -type d -exec chmod 755 {} \;
    sudo chown -R root:root pack
    sudo dpkg-deb --build pack; mv pack.deb ../../debian/greadymedia_"$version"_"$arch".deb
    sudo chown -R $USER pack
    cd ../..; rm -rf greadymedia-$version
}


changelog_version=$(cat doc/ChangeLog.md | grep "## " | head -n1 | sed 's/.* - v//')
cmake_version=$(cat CMakeLists.txt | grep "(PROJECT_VERSION \"" | cut -f2 -d\")
dir_version=$(basename $(pwd) | rev | cut -f1 -d- |rev)
if [ "$dir_version" != "$cmake_version" ] || [ "$dir_version" != "$changelog_version" ]; then
    echo "version error: dir=$dir_version CMakeLists.txt=$cmake_version, doc/ChangeLog.md=$changelog_version" >&2
    exit 1
fi
version=$dir_version

rm -rf releases www
cd ..; tar -czvf greadymedia-$version.tar.gz greadymedia-$version; cd greadymedia-$version
mkdir -p releases/{arch,slackware,debian}
mv ../greadymedia-$version.tar.gz releases/

cd releases
make_arch_package $version
make_slackware_package $version
make_debian_package $version
cd ..

gpg --detach-sign --sign $(ls releases/*.tar.gz)
gpg --detach-sign --sign $(ls releases/arch/*)
gpg --detach-sign --sign $(ls releases/debian/*)
gpg --detach-sign --sign $(ls releases/slackware/*)

if [ "$1" == "-u" ]; then
    ssh -4 -t root@fox-echo.info "mkdir -p /var/www/fox-echo.info/greadymedia"
    scp -4 -r releases/* root@fox-echo.info:/var/www/fox-echo.info/greadymedia/res/
    sitegen.sh
    scp -4 www/* root@fox-echo.info:/var/www/fox-echo.info/greadymedia/
fi
