#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <locale.h>
#include <signal.h>
#include "greadymedia.h"
#include "config.h"


int main (int argc, char *argv[]) {
	setlocale (LC_ALL, "");
	bindtextdomain (PROJECT_NAME, LOCALE_DIR);
	bind_textdomain_codeset (PROJECT_NAME, "UTF-8");
	textdomain (PROJECT_NAME);
	
	app = g_new (T_APP, 1);
	app->user = g_getenv ("USER");
	app->configdir = g_strdup_printf ("%s/.config/minidlna", g_getenv ("HOME"));
	if (g_mkdir_with_parents (app->configdir, DIR_MODE) != 0) {
		g_print (gettext ("error: can't create directory '%s'\n"), app->configdir);
		return 1;
	}
	app->statedir = g_strdup_printf ("%s/.local/state/minidlna", g_getenv ("HOME"));
	if (g_mkdir_with_parents (app->configdir, DIR_MODE) != 0) {
		g_print (gettext ("error: can't create directory '%s'\n"), app->statedir);
		return 1;
	}
	
	GtkApplication *application = gtk_application_new ("fr.endres.greadymedia", G_APPLICATION_DEFAULT_FLAGS);
	g_signal_connect (application, "activate", (GCallback) on_activate, NULL);
	gint status = g_application_run ((GApplication *) application, argc, argv);
	g_object_unref (application);
	return status;
}


gint get_server_pid() {
	gint pid = 0;
	gchar *pidpath = g_strdup_printf ("%s/minidlna.pid", app->statedir);
	if (g_file_test (pidpath, G_FILE_TEST_EXISTS)) {
		GFile *pidfile = g_file_new_for_path (pidpath);
		gchar *content;
		g_file_load_contents (pidfile, NULL, &content, NULL, NULL, NULL);
		content[strlen (content) - 1] = '\0'; //remove trailing '\n'
		pid = g_ascii_strtoull (content, NULL, 10);
		g_free (content);
		g_object_unref (pidfile);
	}
	g_free (pidpath);
	return pid;
}


gboolean is_duplicate (const gchar *newFolder) {
	GtkTreeIter iter;
	gboolean iterate = gtk_tree_model_get_iter_first ((GtkTreeModel *) gui->folder_model, &iter);
	gboolean duplicated = FALSE;
	while (iterate && !duplicated) {
		gchar *path;
		gtk_tree_model_get ((GtkTreeModel *) gui->folder_model, &iter, 0, &path, -1);
		if (g_ascii_strcasecmp (newFolder, path) == 0) duplicated = TRUE;
		g_free (path);
		iterate = gtk_tree_model_iter_next ((GtkTreeModel *) gui->folder_model, &iter);
	}
	return duplicated;
}


void load_shares () {
	gchar *content;
	gchar *filepath = g_strdup_printf ("%s/minidlna.conf", app->configdir);
	if (g_file_get_contents (filepath, &content, NULL, NULL)) {
		gchar **lines = g_strsplit (content, "\n", 0);
		g_free (content);
			for (gushort i=0; lines[i] != NULL; i++) {
				if (0 == g_ascii_strncasecmp ("media_dir=", lines[i], strlen ("media_dir="))) {
					GtkTreeIter iter;
					const gchar *folder = lines[i] + strlen ("media_dir=");
					gtk_list_store_append (gui->folder_model, &iter);
					gtk_list_store_set (gui->folder_model, &iter, 0, folder, -1);
				}
			}
		g_strfreev (lines);
	}
	g_free (filepath);
}


void write_config() {
	GtkTreeIter iter;
	gchar *filepath = g_strdup_printf ("%s/minidlna.conf", app->configdir);
	GFile *configfile = g_file_new_for_path (filepath);
	GFileOutputStream *outputstream = g_file_replace (configfile, NULL, FALSE, G_FILE_CREATE_REPLACE_DESTINATION, NULL, NULL);
	g_output_stream_printf ((GOutputStream *) outputstream, NULL, NULL, NULL, "user=%s\n", app->user);
	g_output_stream_printf ((GOutputStream *) outputstream, NULL, NULL, NULL, "db_dir=%s\n", app->statedir);
	g_output_stream_printf ((GOutputStream *) outputstream, NULL, NULL, NULL, "log_dir=%s\n", app->statedir);
	g_output_stream_printf ((GOutputStream *) outputstream, NULL, NULL, NULL, "log_level=off\n");
	g_output_stream_printf ((GOutputStream *) outputstream, NULL, NULL, NULL, "inotify=yes\n");
	//g_output_stream_printf((GOutputStream *)outputstream, NULL, NULL, NULL, "strict_dlna=no\n", app->configdir);
	gboolean iterate = gtk_tree_model_get_iter_first ((GtkTreeModel *) gui->folder_model, &iter);
	while (iterate) {
		gchar *sharepath;
		gtk_tree_model_get ((GtkTreeModel *) gui->folder_model, &iter, 0, &sharepath, -1);
		g_output_stream_printf ((GOutputStream *) outputstream, NULL, NULL, NULL, "media_dir=%s\n", sharepath);
		g_free(sharepath);
		iterate = gtk_tree_model_iter_next ((GtkTreeModel *) gui->folder_model, &iter);
	}
	g_output_stream_close ((GOutputStream *) outputstream, NULL, NULL);
	g_object_unref (outputstream);
	g_object_unref (configfile);
	g_free (filepath);
}


void on_activate (GApplication *application, gpointer user_data) {
	gui = g_new(T_GUI, 1);
	gui->builder = gtk_builder_new();
	gtk_builder_add_from_file(gui->builder, UI_FILE, NULL);
	
	gui->main_application =(GtkApplicationWindow *) gtk_builder_get_object(gui->builder, "main_application");
	gtk_window_set_icon_name((GtkWindow *) gui->main_application, PROJECT_NAME);
	gui->about_dialog =(GtkAboutDialog *) gtk_builder_get_object(gui->builder, "about_dialog");
	gtk_window_set_icon_name((GtkWindow *) gui->about_dialog, PROJECT_NAME);
	gtk_about_dialog_set_version(gui->about_dialog, PROJECT_VERSION);
	
	gui->folder_view =(GtkTreeView *) gtk_builder_get_object(gui->builder, "folder_view");
	gui->folder_model =(GtkListStore *) gtk_builder_get_object(gui->builder, "folder_model");
	gui->on_off =(GtkSwitch *) gtk_builder_get_object(gui->builder, "on_off");
	
	gchar *proc = g_strdup_printf("/proc/%d", get_server_pid());
	if (g_file_test(proc, G_FILE_TEST_IS_DIR)) { //if server is already active
		gtk_switch_set_state(gui->on_off, TRUE);
	}
	g_free(proc);
	load_shares();
	
	gtk_builder_connect_signals(gui->builder, NULL);
	gtk_window_set_application ((GtkWindow *) gui->main_application, (GtkApplication *) application);
	gtk_widget_show((GtkWidget *) gui->main_application);
}


gboolean on_state_change (GtkSwitch* sw, gboolean state, gpointer user_data) {
	if (TRUE == state) {
		GtkTreeIter iter;
		if (gtk_tree_model_get_iter_first ((GtkTreeModel *) gui->folder_model, &iter)) {
			gchar *command = g_strdup_printf ("minidlnad -f %s/minidlna.conf -P %s/minidlna.pid", app->configdir, app->statedir);
			g_spawn_command_line_async (command, NULL);
			g_free (command);
			gtk_switch_set_state (sw, TRUE);
		} else {
			gtk_switch_set_active (sw, FALSE);
		}
	} else {
		gtk_switch_set_state (sw, FALSE);
		gint pid = get_server_pid();
		if (pid != 0) {
			kill (pid, SIGTERM);
		}
	}
	return TRUE;
}


void on_add_folder (GtkButton *bt, gpointer user_data) {
	GtkFileChooser *filechooser = (GtkFileChooser *) gtk_file_chooser_dialog_new (gettext("Choose shared folder"), (GtkWindow *) gui->main_application, 
			GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, gettext("_Cancel"), GTK_RESPONSE_CANCEL, gettext("_Select"), GTK_RESPONSE_ACCEPT, NULL);
	if (gtk_dialog_run ((GtkDialog *) filechooser) == GTK_RESPONSE_ACCEPT) {
		GFile *file = gtk_file_chooser_get_file (filechooser);
		gchar *folder = g_file_get_path (file);
		if (!is_duplicate(folder)) {
			GtkTreeIter iter;
			gtk_list_store_append (gui->folder_model, &iter);
			gtk_list_store_set (gui->folder_model, &iter, 0, folder, -1);
		}
		g_free (folder);
		g_object_unref (file);
		write_config();
	}
	gtk_widget_destroy ((GtkWidget *) filechooser);
}


void on_remove_folder (GtkButton *bt, gpointer user_data) {
	GtkTreeIter iter;
	GtkTreeSelection *selection = gtk_tree_view_get_selection (gui->folder_view);
	if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
		gtk_list_store_remove (gui->folder_model, &iter);
	}
	write_config();
}
